@@ -11,18 +11,18 @@ use List::Util qw(shuffle);
 our $AUTHORITY = 'cpan:PERLANCAR'; # AUTHORITY
 our $DATE = '2022-08-20'; # DATE
 our $DIST = 'App-wordlist'; # DIST
-our $VERSION = '0.285'; # VERSION
+our $VERSION = '0.286'; # VERSION
 
 our %SPEC;
 
 our %argspecopt_wordlists = (
     wordlists => {
+        summary => 'Select one or more wordlist modules',
         'x.name.is_plural' => 1,
         schema => ['array*' => {
             of => 'str*', # for the moment we need to use 'str' instead of 'perl::modname' due to Perinci::Sub::GetArgs::Argv limitation
             'x.perl.coerce_rules'=>[ ['From_str_or_array::expand_perl_modname_wildcard'=>{ns_prefix=>"WordList"}] ],
         }],
-        summary => 'Select one or more wordlist modules',
         cmdline_aliases => {w=>{}},
         element_completion => sub {
             require Complete::Util;
@@ -38,6 +38,40 @@ our %argspecopt_wordlists = (
     },
 );
 
+our %argspecsopt_exclude_wordlist = (
+    exclude_wordlists => {
+        'x.name.is_plural' => 1,
+        'x.name.singular' => 'exclude_wordlist',
+        summary => 'Exclude wordlist modules',
+        schema => ['array*' => {
+            of => 'str*', # for the moment we need to use 'str' instead of 'perl::modname' due to Perinci::Sub::GetArgs::Argv limitation
+            'x.perl.coerce_rules'=>[ ['From_str_or_array::expand_perl_modname_wildcard'=>{ns_prefix=>"WordList"}] ],
+        }],
+        element_completion => sub {
+            require Complete::Util;
+
+            my %args = @_;
+            Complete::Util::complete_array_elem(
+                word  => $args{word},
+                array => [map {$_->{name}} @{ _list_installed() }],
+                ci    => 1,
+            );
+        },
+        cmdline_aliases => {X=>{}},
+        tags => ['category:module-selection'],
+    },
+    exclude_wordlist_pattern => {
+        schema => 're_from_str*',
+        cmdline_aliases => {P=>{}},
+        tags => ['category:module-selection'],
+    },
+    exclude_dynamic_wordlists => {
+        schema => 'bool*',
+        cmdline_aliases => {D=>{}},
+        tags => ['category:module-selection'],
+    },
+);
+
 our %argspecopt_wordlist_bundles = (
     wordlist_bundles => {
         'x.name.is_plural' => 1,
@@ -161,7 +195,7 @@ sub _word_has_chars_ordered {
 
 $SPEC{wordlist} = {
     v => 1.1,
-    summary => 'Grep words from WordList::*',
+    summary => 'Grep words from (or test them against) WordList::*',
     args => {
         arg => {
             schema => ['array*' => of => 'str*'],
@@ -207,6 +241,9 @@ _
 
         %argspecopt_wordlists,
         %argspecopt_wordlist_bundles,
+
+        %argspecsopt_exclude_wordlist,
+
         or => {
             summary => 'Match any word in query instead of the default "all"',
             schema  => 'bool',
@@ -333,6 +370,12 @@ _
             test => 0,
             'x.doc.show_result' => 0,
         },
+        {
+            argv => [qw/-D/],
+            summary => 'Print all words from all static wordlists (dynamic ones are excluded with --exclude-dynamic-wordlists a.k.a. -D)',
+            test => 0,
+            'x.doc.show_result' => 0,
+        },
         {
             argv => [qw/foo bar/],
             summary => 'Print all words matching /foo/ and /bar/',
@@ -438,7 +481,21 @@ _
             summary => 'Check some passwords against a password wordlist',
             test => 0,
             'x.doc.show_result' => 0,
-            tags => ['category:action-stat'],
+            tags => ['category:action-test'],
+        },
+        {
+            argv => [qw/-w Password::** -X Password::RockYou -t foobar 123456 someGoodPass923/],
+            summary => 'Check some passwords against all Password wordlists except Password::RockYou (because it is slow to check)',
+            test => 0,
+            'x.doc.show_result' => 0,
+            tags => ['category:action-test'],
+        },
+        {
+            argv => [qw/-w Password::** -P RockYou -t foobar 123456 someGoodPass923/],
+            summary => 'Check some passwords against all Password wordlists except those matching /RockYou/ regex',
+            test => 0,
+            'x.doc.show_result' => 0,
+            tags => ['category:action-test'],
         },
     ],
     'cmdline.default_format' => 'text-simple',
@@ -560,6 +617,44 @@ sub wordlist {
             }
         }
 
+      EXCLUDE_WORDLISTS: {
+            if ($args{exclude_wordlists} && @{ $args{exclude_wordlists} }) {
+                my $filtered_wordlists = [];
+              WORDLIST:
+                for my $wl (@$wordlists) {
+                    for my $exwl (@{ $args{exclude_wordlists} }) {
+                        do { log_trace "Excluding wordlist $wl (--excluded-wordlist)"; next WORDLIST } if $wl eq $exwl;
+                    }
+                    push @$filtered_wordlists, $wl;
+                }
+                $wordlists = $filtered_wordlists;
+            }
+
+            if (defined $args{exclude_wordlist_pattern}) {
+                my $filtered_wordlists = [];
+              WORDLIST:
+                for my $wl (@$wordlists) {
+                    do { log_trace "Excluding wordlist $wl (--excluded-wordlist-pattern)"; next WORDLIST } if $wl =~ $args{exclude_wordlist_pattern};
+                    push @$filtered_wordlists, $wl;
+                }
+                $wordlists = $filtered_wordlists;
+            }
+
+            if ($args{exclude_dynamic_wordlists}) {
+                no strict 'refs'; ## no critic: TestingAndDebugging::ProhibitNoStrict
+                my $filtered_wordlists = [];
+              WORDLIST:
+                for my $wl (@$wordlists) {
+                    my $mod = "WordList::$wl"; $mod =~ s/=.*//;
+                    (my $modpm = "$mod.pm") =~ s!::!/!g;
+                    require $modpm;
+                    do { log_trace "Excluding wordlist $wl (--excluded-dynamic-wordlists)"; next WORDLIST } if ${"$mod\::DYNAMIC"};
+                    push @$filtered_wordlists, $wl;
+                }
+                $wordlists = $filtered_wordlists;
+            }
+        }
+
         $wordlists = [shuffle @$wordlists] if $random;
         log_trace "Wordlist(s) to use: %s", $wordlists;
 
@@ -568,9 +663,10 @@ sub wordlist {
         }
 
         if ($action eq 'stat') {
-            no strict 'refs';
+            no strict 'refs'; ## no critic: TestingAndDebugging::ProhibitNoStrict
             return [200] unless @$wordlists;
             my %all_stats;
+          WORDLIST:
             for my $wl (@$wordlists) {
                 my $mod = "WordList::$wl"; $mod =~ s/=.*//;
                 (my $modpm = "$mod.pm") =~ s!::!/!g;
@@ -591,6 +687,7 @@ sub wordlist {
           WORDLIST:
             for my $wl (@$wordlists) {
                 last unless @words;
+                log_debug "Testing against wordlist $wl ...";
                 my $wl_obj;
                 eval {
                     $wl_obj = Module::Load::Util::instantiate_class_with_optional_args(
@@ -786,7 +883,7 @@ sub wordlist {
 }
 
 1;
-# ABSTRACT: Grep words from WordList::*
+# ABSTRACT: Grep words from (or test them against) WordList::*
 
 __END__
 
@@ -796,11 +893,11 @@ __END__
 
 =head1 NAME
 
-App::wordlist - Grep words from WordList::*
+App::wordlist - Grep words from (or test them against) WordList::*
 
 =head1 VERSION
 
-This document describes version 0.285 of App::wordlist (from Perl distribution App-wordlist), released on 2022-08-20.
+This document describes version 0.286 of App::wordlist (from Perl distribution App-wordlist), released on 2022-08-20.
 
 =head1 SYNOPSIS
 
@@ -815,7 +912,7 @@ Usage:
 
  wordlist(%args) -> [$status_code, $reason, $payload, \%result_meta]
 
-Grep words from WordList::*.
+Grep words from (or test them against) WordList::*.
 
 Examples:
 
@@ -825,6 +922,10 @@ Examples:
 
  wordlist();
 
+=item * Print all words from all static wordlists (dynamic ones are excluded with --exclude-dynamic-wordlists a.k.a. -D):
+
+ wordlist(exclude_dynamic_wordlists => 1);
+
 =item * Print all words matching E<sol>fooE<sol> and E<sol>barE<sol>:
 
  wordlist(arg => ["foo", "bar"]);
@@ -907,6 +1008,24 @@ Examples:
    wordlists => ["Password::RockYou::BloomOnly"]
  );
 
+=item * Check some passwords against all Password wordlists except Password::RockYou (because it is slow to check):
+
+ wordlist(
+     arg => ["foobar", 123456, "someGoodPass923"],
+   action => "test",
+   exclude_wordlists => ["Password::RockYou"],
+   wordlists => ["Password::**"]
+ );
+
+=item * Check some passwords against all Password wordlists except those matching E<sol>RockYouE<sol> regex:
+
+ wordlist(
+     arg => ["foobar", 123456, "someGoodPass923"],
+   action => "test",
+   exclude_wordlist_pattern => "RockYou",
+   wordlists => ["Password::**"]
+ );
+
 =back
 
 This function is not exported.
@@ -959,6 +1078,14 @@ language.
 
 When returning grep result, this means also returning wordlist name.
 
+=item * B<exclude_dynamic_wordlists> => I<bool>
+
+=item * B<exclude_wordlist_pattern> => I<re_from_str>
+
+=item * B<exclude_wordlists> => I<array[str]>
+
+Exclude wordlist modules.
+
 =item * B<ignore_case> => I<bool> (default: 1)
 
 =item * B<langs> => I<array[str]>
